import sys
try:
    import psyco
    psyco.full()
except:
	print 'psyco not running'
from cistematic.core.motif import Motif
from cistematic.core import complement
from cistematic.genomes import Genome
from commoncode import *

print '%s: version 2.2' % sys.argv[0]
if len(sys.argv) < 6:
    print 'usage: python %s genome motifFile motThreshold regionfile siteOutfile [-dataset chipRDS] [-cache] [-printseq] [-nomerge] [-markov1] ' % sys.argv[0]
    sys.exit(1)

genome = sys.argv[1]
motfilename = sys.argv[2]
motThreshold = float(sys.argv[3])

doMarkov1 = False
if '-markov1' in sys.argv:
    doMarkov1 = True

if motThreshold < 1.0 and doMarkov1:
    print 'motThreshold should be between 1.0 and 10.0 for markov1'
    sys.exit(1)
elif motThreshold < 55.0 and not doMarkov1:
    print 'motThreshold should be between 55 and 99 for a regular PSFM'
    sys.exit(1)

#pvalfilename = 'pvalueunamp.txt'
infilename = sys.argv[4]
outfilename = sys.argv[5]

doRDS = False
if '-dataset' in sys.argv:
    chipfilename = sys.argv[sys.argv.index('-dataset') + 1]
    doRDS = True

doCache = False
if '-cache' in sys.argv:
    doCache = True

printSeq = False
if '-printseq' in sys.argv:
    printSeq = True
maxPvalue = 0.0001

mot = Motif('',motifFile = motfilename)
motLen = len(mot)
bestScore = mot.bestConsensusScore()

hg = Genome(genome)

# minHits=-1 will force regions to be used regardless
# maxDist= 0 prevents merging of non-overlapping regions
if '-nomerge' in sys.argv:
    regions = getMergedRegions(infilename, maxDist=0, minHits=-1, verbose=True, doMerge=False)
else:
    regions = getMergedRegions(infilename, maxDist=0, minHits=-1, verbose=True)

if doRDS:
    hitRDS = readDataset(chipfilename, verbose = True, cache=doCache)

outfile = open(outfilename,'w')

index = 0
regionList = []

for chrom in regions:
    if 'rand' in chrom or 'M' in chrom:
        continue
    for (start, stop, length) in regions[chrom]:
        regionList.append((chrom, start, length))
	
notFoundIndex = 0
currentChrom = ''
count = 0
for (rchrom, start, length) in regionList:
    try:
        seq = hg.sequence(rchrom, start, length)
    except:
        print "couldn't retrieve %s %d %d - skipping" % (rchrom, start, length)
        continue
    numHits = -1
    if doRDS:
        if rchrom != currentChrom:
            fullchrom = 'chr' + rchrom
            hitDict = hitRDS.getReadsDict(chrom=fullchrom)
            currentChrom = rchrom
        (topPos, numHits, smoothArray, numPlus) = findPeak(hitDict[rchrom], start, length)
        if len(topPos) == 0:
            print 'topPos error'
        peakpos = topPos[0]
    found = []
    if doMarkov1:
        matches = mot.locateMarkov1(seq, motThreshold)
    else:
        matches = mot.locateMotif(seq, motThreshold)
    for (pos, sense) in matches:
        alreadyFound = False
        for (fpos, fdist) in found:
            if pos + start == fpos:
                alreadyFound = True
        if not alreadyFound:
            if doRDS:
                found.append((start + pos, pos  + motLen/2 - peakpos))
            else:
                found.append((start + pos, -1))
                            
    foundValue = False
    for (foundpos, peakdist) in found:
        seq = hg.sequence(rchrom, foundpos, motLen)
        foundValue = True
        (front, back) = mot.scoreMotif(seq)
        sense = '+'
        if front >= back:
            score = int(100 * front / bestScore)
        else:
            score = int(100 * back / bestScore)
            sense = '-'
            #foundpos += 1
            seq = complement(seq)
        if printSeq:
            print seq
        outfile.write('chr%s:%d-%d\t%d\t%d\t%d\tchr%s:%d-%d\t%s\n' % (rchrom, foundpos, foundpos + motLen - 1, score, numHits, peakdist, rchrom, start, start + length, sense))
    if not foundValue:
        if printSeq:
            print 'could not find a %s site for %s:%d-%d' % (mot.tagID, rchrom, start, start+ length)
        notFoundIndex += 1
    count += 1
    if (count % 100000) == 0 and not printSeq:
        print count
outfile.close()
print 'did not find motif in %d regions' % notFoundIndex
