#
#  regionCounts.py
#  ENRAGE
#

try:
    import psyco
    psyco.full()
except:
    print 'psyco not running'

import sys
print '%s: version 3.2' % sys.argv[0]

if len(sys.argv) < 4:
    print 'usage: python %s regionfile rdsfile outfilename [-markRDS] [-chromField fieldNum] [-raw] [-padregion bp] [-mergeregion bp] [-noUniqs] [-noMulti] [-splices] [-peak] [-cache]' % sys.argv[0]
    sys.exit(1)

from commoncode import *

regionfilename = sys.argv[1]
hitfile =  sys.argv[2]
outfilename = sys.argv[3]

flagRDS = False
if '-markRDS' in sys.argv:
    flagRDS = True

cField = 1
if '-chromField' in sys.argv:
        cField = int(sys.argv[sys.argv.index('-chromField') + 1])

if '-raw' in sys.argv:
    normalize = False
else:
    normalize = True

padregion = 0
if '-padregion' in sys.argv:
    padField = sys.argv.index('-padregion') + 1
    padregion = int(sys.argv[padField])
    print 'padding %d bp on each side of a region' % padregion

mergeregion = 0
if '-mergeregion' in sys.argv:
    mergeField = sys.argv.index('-mergeregion') + 1
    mergeregion = int(sys.argv[mergeField])
    print 'merging regions closer than %d bp' % mergeregion

usePeak = False
if '-peak' in sys.argv:
    print 'will use peak values'
    usePeak = True

doUniqs = True
doMulti = True
doSplices = False
if '-noUniqs' in sys.argv:
    doUniqs = False
if '-noMulti' in sys.argv:
    doMulti = False
if '-splices' in sys.argv:
    doSplices = True

doCache = False
if '-cache' in sys.argv:
    doCache = True

regionDict = getMergedRegions(regionfilename, maxDist = mergeregion,  minHits=-1, keepLabel = True, verbose = True, chromField = cField, pad=padregion)
labelList = []
labeltoRegionDict = {}
regionCount = {}

hitRDS = readDataset(hitfile, verbose = True, cache=doCache)
readlen = hitRDS.getReadSize()
if normalize:
    totalCount = len(hitRDS)
    normalizationFactor = totalCount / 1000000.

chromList = hitRDS.getChromosomes(fullChrom=False)
chromList.sort()

if flagRDS:
    hitRDS.setSynchronousPragma('OFF')        

index = 0
for rchrom in chromList:
    regionList = []
    if rchrom not in regionDict:
        continue
    print rchrom
    if usePeak:
        readDict = hitRDS.getReadsDict(chrom=fullchrom, withWeight=True, doMulti=True, findallOptimize=True)
        rindex = 0
        dictLen = len(readDict[fullchrom])
    
    for (label, start, stop, length) in regionDict[rchrom]:
        regionCount[label] = 0
        labelList.append(label)
        labeltoRegionDict[label] = (rchrom, start, stop)
        
    fullchrom = 'chr' + rchrom
    for (label, rstart, rstop, length) in regionDict[rchrom]:
        regionList.append((label, fullchrom, rstart, rstop))
        if usePeak:
            readList = []
            for localIndex in xrange(rindex, dictLen):
                read = readDict[fullchrom][localIndex]
                if read[0] < rstart:
                    rindex += 1
                elif rstart <= read[0] <= rstop:
                    readList.append(read)
                else:
                    break
            if len(readList) < 1:
                continue
            readList.sort()
            (topPos, numHits, smoothArray, numPlus) = findPeak(readList, rstart, rstop - rstart, readlen, doWeight=True)
            try:
                topValue = smoothArray[topPos[0]]
            except:
                print "problem with %s %s" % (str(topPos), str(smoothArray))
                continue
            regionCount[label] += topValue
        else:
            regionCount[label] += hitRDS.getCounts(fullchrom, rstart, rstop, uniqs=doUniqs, multi=doMulti, splices=doSplices)
    if flagRDS:
        hitRDS.flagReads(regionList, uniqs=doUniqs, multi=doMulti, splices=doSplices)
if flagRDS:
    hitRDS.setSynchronousPragma('ON')
            

#infile = open(infilename)
if normalize:
    for label in regionCount:
        regionCount[label] = float(regionCount[label]) / normalizationFactor

outfile = open(outfilename,'w')


for label in labelList:
    (chrom, start, stop) = labeltoRegionDict[label]
    if normalize:
        outfile.write('%s\tchr%s\t%d\t%d\t%.2f\n' % (label, chrom, start, stop, regionCount[label]))
    else:
        outfile.write('%s\tchr%s\t%d\t%d\t%d\n' % (label, chrom, start, stop, regionCount[label]))

#infile.close()
outfile.close()
if doCache and flagRDS:
    hitRDS.saveCacheDB(hitfile)


